"""
Entrypoint module, in case you use `python -mhelius_sdk`.
"""
import pandas as pd
from helius_sdk.core import CONFIG
from helius_sdk.event import Sale, Mint, Listing
from helius_sdk.nft import Nft
from helius_sdk.transaction import Transaction

COL_ADDRESS = "4mKSoDDqApmF1DqXvVTSL6tu2zixrSSNjqMxUnwvVzy2"
WALLET_ADDRESS = "GYpFo2YSBQMZefhR9bTTTx7YAugvgKyodX7tUG1pDCux"
TX_ADDRESS = "4Mv6CFqtYTdCf1ShunYStgrvFBEZFJSfwjTRc4asmXYdZsJgQ15RgUSU6xZQACtYTyDWSrNH9eEuCF5Ypf9zek89"

def main():
    sale = Sale(CONFIG["api_key"], col_address="")
    s = sale.get()

    listing = Listing(CONFIG["api_key"], col_address=COL_ADDRESS)
    l = listing.get()
    print(s.columns)
    pd.set_option("display.max_colwidth", 100)
    print(s.head().to_pandas().iloc[0])
    print(s.head())

    pd.set_option("display.max_colwidth", 100)

    mint = Mint(api_key=CONFIG["api_key"], wallet_address=WALLET_ADDRESS)
    m = mint.get()
    print(m.shape)

    n = Nft(api_key=CONFIG["api_key"], col_address="KG6f4Fa6YxAW8cG2Dhb18DiMn3rQ3rSLa1Eo2FYM4gi").get()
    print(n.shape)

    t = Transaction(api_key=CONFIG["api_key"],
                    tx_address=TX_ADDRESS).get()
    print(t)


if __name__ == "__main__":
    main()



from abc import ABC
from helius_sdk.core import HeliusAPI


class NftAPI(HeliusAPI, ABC):
    """
    Get events data from Helius API.
    """

    def __init__(self, api_key=None, query=None, col_address=None, sources=None):
        """

        :param api_key:
        :param query:
        :param col_address:
        :param sources:
        """
        super().__init__(api_key=api_key, slug="nfts")
        self.sources = sources
        self.col_address = col_address
        self.query = query

    def get_query(self, qtype: [str], kwargs={}):
        """
        Build query for Helius API
        :param qtype:
        :param kwargs:
        :return:
        """
        q = {"types": qtype, "nftCollectionFilters": {"verifiedCollectionAddress": [self.col_address]}, **kwargs}
        if self.sources:
            q["sources"] = self.sources or ["MAGIC_EDEN"]
        return q


class Nft(NftAPI):
    """
    Get sales data from Helius API.

    example:
    >>> from helius_sdk.nft import Nft
    >>> n = Nft(api_key="api_key", col_address="").get()
    >>> n.shape
    """
    def get(self, query=None, df=True):
        q = query or {"mints": [self.col_address]}
        return self.make_request(q, df)



import os
from abc import ABC, abstractmethod

import requests
import yaml
import polars as pl


def load_config(config_path="conf/base.yml"):
    if os.path.exists(config_path):
        with open(config_path, "r") as config_file:
            config = yaml.safe_load(config_file)
        return config["helius"]
    else:
        return {
            "base_url": os.getenv("BASE_URL") or "https://api.helius.xyz/v1",
            "api_key": os.getenv("API_KEY"),
        }


CONFIG = load_config()


class HeliusAPI(ABC):
    BASE_URL = CONFIG["base_url"]

    def __init__(self, api_key, slug=None):
        self.api_key = api_key or CONFIG["api_key"]
        if os.getenv("API_KEY"):
            self.api_key = os.getenv("API_KEY")

        if not self.api_key:
            raise ValueError("No API key provided")
        self.query = None
        self.url = f"{self.BASE_URL}/{slug}?api-key={self.api_key}" if slug else None

    @abstractmethod
    def get(self, query=None):
        pass

    def make_request(self, query, df=True):
        """
        Make post request to Helius API
        :param query:
        :param df:
        :return:
        """
        try:
            response = requests.post(self.url, json=query)
            if not df:
                return response.json()
            return to_df(response.json())
        except Exception as e:
            print(e)
            raise e


def to_df(data, pandas=False):
    if "result" in data:
        data = data["result"]
    if pandas:
        import pandas as pd
        return pd.DataFrame(data)
    return pl.DataFrame(data)



from abc import ABC

import requests
from helius_sdk.core import HeliusAPI, to_df, CONFIG

YOOTS_ADDRESS = "4mKSoDDqApmF1DqXvVTSL6tu2zixrSSNjqMxUnwvVzy2"


class EventsAPI(HeliusAPI, ABC):
    """
    Get events data from Helius API.
    """

    def __init__(self, api_key=None, query=None, col_address=None, sources=None):
        """

        :param api_key:
        :param query:
        :param col_address:
        :param sources:

        e.g.:
        query = {
              "query": {
                "accounts": [
                  "79FjnnXFpSajugwpx3FfQbV7hJJTe7QXAusF7R4CMf6c"
                ],
                "types": [
                  "NFT_SALE"
                ],
                "sources": [
                  "MAGIC_EDEN"
                ],
                "startSlot": 162368570,
                "endSlot": 162368580,
                "startTime": 1669055189,
                "endTime": 1669070189,
                "nftCollectionFilters": {
                  "firstVerifiedCreator": [
                    "A4FM6h8T5Fmh9z2g3fKUrKfZn6BNFEgByR8QGpdbQhk1"
                  ],
                  "verifiedCollectionAddress": [
                    "4mKSoDDqApmF1DqXvVTSL6tu2zixrSSNjqMxUnwvVzy2"
                  ]
                }
              },
              "options": {
                "limit": 100,
                "sortOrder": "DESC",
                "paginationToken": "V1_162262948_99"
              }
            }
        """
        # super().__init__(api_key, )
        super().__init__(api_key=api_key, slug="nft-events")
        self.sources = sources
        self.col_address = col_address or YOOTS_ADDRESS
        # self.url = f"{self.BASE_URL}/nft-events?api-key={self.api_key}"
        self.query = query

    # def make_request(self, query, df=True):
    #     """
    #     Make post request to Helius API
    #     :param query:
    #     :param df:
    #     :return:
    #     """
    #     try:
    #         response = requests.post(self.url, json={"query": query})
    #         if not df:
    #             return response.json()
    #         return to_df(response.json())
    #     except Exception as e:
    #         raise e

    def get_query(self, qtype: [str], kwargs={}):
        """
        Build query for Helius API
        :param qtype:
        :param kwargs:
        :return:
        """
        q = {"types": qtype, "nftCollectionFilters": {"verifiedCollectionAddress": [self.col_address]}, **kwargs}
        if self.sources:
            q["sources"] = self.sources or ["MAGIC_EDEN"]
        return {"query": q}


class Sale(EventsAPI):
    """
    Get sales data from Helius API.

    example:
    >>> from helius_sdk.event import Sale
    >>> sale = Sale().get()
    >>> sale.shape
    """
    def get(self, query=None, df=True):
        q = query or self.get_query(["NFT_SALE"])
        return self.make_request(q, df)


class Listing(EventsAPI):
    """
    Get listings data from Helius API.

    example:
    >>> from helius_sdk.event import Listing
    >>> l = Listing().get()
    >>> l.shape

    """
    def get(self, query=None, df=True):
        q = query or self.get_query(["NFT_LISTING"])
        return self.make_request(q, df)


class Mint(EventsAPI):
    """
    Get mints data from Helius API.

    example:
    >>> from helius_sdk.event import Mint
    >>> m = Mint().get()
    >>> m.shape
    """
    def __init__(self, wallet_address=None, col_address=None, api_key=None, query=None, sources=None):
        super().__init__(api_key, query, col_address, sources)
        self.wallet_address = wallet_address

    def get(self, query=None, df=True):
        q = query or self.get_query(["NFT_MINT"], kwargs={"accounts": [self.wallet_address]})
        return self.make_request(q, df)


# Create a higher level API
# def get_sales(query=None):
#     sale = Sale(api_key=CONFIG["api_key"])
#     return sale.get(query)


# def get_mints(self, query=None):
#     sale = Sale(api_key=CONFIG["api_key"])
#     return Mint(self.api_key).get(query)
#
#
# def get_listings(self, query=None):
#     sale = Sale(api_key=CONFIG["api_key"])
#     return Listing(self.api_key).get(query)



from abc import ABC
from helius_sdk.core import HeliusAPI


class TransactionAPI(HeliusAPI, ABC):
    """
    Get events data from Helius API.
    """

    def __init__(self, api_key=None, query=None, tx_address=None):
        """

        :param api_key:
        :param query:
        :param tx_address:
        """
        super().__init__(api_key=api_key, slug="transactions")
        self.query = query
        self.url = self.url.replace("v1", "v0")
        # f"{self.BASE_URL}/v0/transactions?api-key={self.api_key}"
        self.tx_address = tx_address


class Transaction(TransactionAPI):
    """
    Get sales data from Helius API.

    example:
    >>> from helius_sdk.transaction import Transaction
    >>> t = Transaction(api_key="api_key", tx_address="").get()
    >>> t.shape
    """
    def get(self, query=None, df=True):
        q = query or {"transactions": [self.tx_address]}
        return self.make_request(q, df)
